/**
 * @popperjs/core v2.0.0-alpha.3
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.Popper = {}));
}(this, (function (exports) { 'use strict';

  var top = 'top';
  var bottom = 'bottom';
  var right = 'right';
  var left = 'left';
  var basePlacements = [top, bottom, right, left];
  var start = 'start';
  var end = 'end';
  var clippingParents = 'clippingParents';
  var viewport = 'viewport';
  var popper = 'popper';
  var reference = 'reference';
  var placements = basePlacements.reduce(function (acc, placement) {
    return acc.concat([placement, placement + "-" + start, placement + "-" + end]);
  }, []); // modifiers that need to read the DOM

  var beforeRead = 'beforeRead';
  var read = 'read';
  var afterRead = 'afterRead'; // pure-logic modifiers

  var beforeMain = 'beforeMain';
  var main = 'main';
  var afterMain = 'afterMain'; // modifier with the purpose to write to the DOM (or write into a framework state)

  var beforeWrite = 'beforeWrite';
  var write = 'write';
  var afterWrite = 'afterWrite';
  var modifierPhases = [beforeRead, read, afterRead, beforeMain, main, afterMain, beforeWrite, write, afterWrite];

  var getBoundingClientRect = (function (element) {
    var rect = element.getBoundingClientRect();
    return {
      width: rect.width,
      height: rect.height,
      top: rect.top,
      right: rect.right,
      bottom: rect.bottom,
      left: rect.left,
      x: rect.left,
      y: rect.top
    };
  });

  function getWindow(node) {
    var ownerDocument = node != null && typeof node === 'object' && node.hasOwnProperty('ownerDocument') ? // $FlowFixMe: assume ownerDocument to be the one we are looking for
    node.ownerDocument : null;
    return ownerDocument != null ? ownerDocument.defaultView : window;
  }

  function getWindowScroll(node) {
    var win = getWindow(node);
    var scrollLeft = win.pageXOffset;
    var scrollTop = win.pageYOffset;
    return {
      scrollLeft: scrollLeft,
      scrollTop: scrollTop
    };
  }

  /*:: declare function isElement(node: mixed): boolean %checks(node instanceof
    Element); */

  function isElement(node) {
    var OwnElement = getWindow(node).Element;
    return node instanceof OwnElement;
  }
  /*:: declare function isHTMLElement(node: mixed): boolean %checks(node instanceof
    HTMLElement); */


  function isHTMLElement(node) {
    var OwnElement = getWindow(node).HTMLElement;
    return node instanceof OwnElement;
  }

  function getHTMLElementScroll(element) {
    return {
      scrollLeft: element.scrollLeft,
      scrollTop: element.scrollTop
    };
  }

  function getNodeScroll(node) {
    if (node === getWindow(node) || !isHTMLElement(node)) {
      return getWindowScroll(node);
    } else {
      return getHTMLElementScroll(node);
    }
  }

  var getScrollSum = (function (scrollParents) {
    return scrollParents.reduce(function (scroll, scrollParent) {
      var nodeScroll = getNodeScroll(scrollParent);
      scroll.scrollTop += nodeScroll.scrollTop;
      scroll.scrollLeft += nodeScroll.scrollLeft;
      return scroll;
    }, {
      scrollTop: 0,
      scrollLeft: 0
    });
  });

  function getComputedStyle(element) {
    return getWindow(element).getComputedStyle(element);
  }

  var toNumber = function toNumber(cssValue) {
    return parseFloat(cssValue) || 0;
  };

  var getBorders = (function (element) {
    var computedStyle = isHTMLElement(element) ? getComputedStyle(element) : {};
    return {
      top: toNumber(computedStyle.borderTop),
      right: toNumber(computedStyle.borderRight),
      bottom: toNumber(computedStyle.borderBottom),
      left: toNumber(computedStyle.borderLeft)
    };
  });

  var getInnerOffsets = function getInnerOffsets(offsetParent) {
    var rect = getBoundingClientRect(offsetParent);
    var borders = getBorders(offsetParent);
    return {
      x: rect.x + borders.left,
      y: rect.y + borders.top
    };
  }; // Returns the composite rect of an element relative to its offsetParent.
  // Composite means it takes into account transforms as well as layout.


  var getCompositeRect = (function (elementOrVirtualElement, offsetParent, isFixed) {
    if (isFixed === void 0) {
      isFixed = false;
    }

    var rect = getBoundingClientRect(elementOrVirtualElement);
    var scrollSum = getScrollSum(isFixed ? [] : [offsetParent]);
    var offsets = isHTMLElement(offsetParent) && !isFixed ? getInnerOffsets(offsetParent) : {
      x: 0,
      y: 0
    };
    var width = rect.width;
    var height = rect.height;
    var x = rect.left + scrollSum.scrollLeft - offsets.x;
    var y = rect.top + scrollSum.scrollTop - offsets.y;
    return {
      width: width,
      height: height,
      x: x,
      y: y
    };
  });

  // Returns the layout rect of an element relative to its offsetParent. Layout
  // means it doesn't take into account transforms.
  var getLayoutRect = (function (element) {
    return {
      x: element.offsetLeft,
      y: element.offsetTop,
      width: element.offsetWidth,
      height: element.offsetHeight
    };
  });

  var getNodeName = (function (element) {
    return element ? (element.nodeName || '').toLowerCase() : null;
  });

  var getParentNode = (function (element) {
    if (getNodeName(element) === 'html') {
      // DocumentElement detectedF
      return element;
    }

    return element.parentNode || // DOM Element detected
    // $FlowFixMe: need a better way to handle this...
    element.host || // ShadowRoot detected
    document.ownerDocument || // Fallback to ownerDocument if available
    document.documentElement // Or to documentElement if everything else fails
    ;
  });

  function getScrollParent(node) {
    if (['html', 'body', '#document'].includes(getNodeName(node))) {
      return node.ownerDocument.body;
    }

    if (isHTMLElement(node)) {
      // Firefox want us to check `-x` and `-y` variations as well
      var _getComputedStyle = getComputedStyle(node),
          overflow = _getComputedStyle.overflow,
          overflowX = _getComputedStyle.overflowX,
          overflowY = _getComputedStyle.overflowY;

      if (/(auto|scroll|overlay)/.test(overflow + overflowY + overflowX)) {
        return node;
      }
    }

    return getScrollParent(getParentNode(node));
  }

  function listScrollParents(element, list) {
    if (list === void 0) {
      list = [];
    }

    var scrollParent = getScrollParent(element);
    var isBody = getNodeName(scrollParent) === 'body';
    var target = isBody ? getWindow(scrollParent) : scrollParent;
    var updatedList = list.concat(target);
    return isBody ? updatedList : updatedList.concat(listScrollParents(getParentNode(target)));
  }

  var isTableElement = (function (element) {
    return ['table', 'td', 'th'].includes(getNodeName(element));
  });

  var isFirefox = function isFirefox() {
    return typeof window.InstallTrigger !== 'undefined';
  };

  var getTrueOffsetParent = function getTrueOffsetParent(element) {
    var offsetParent;

    if (!isHTMLElement(element) || !(offsetParent = element.offsetParent) || // https://github.com/popperjs/popper.js/issues/837
    isFirefox() && getComputedStyle(offsetParent).position === 'fixed') {
      return null;
    }

    return offsetParent;
  };

  function getOffsetParent(element) {
    var window = getWindow(element);
    var offsetParent = getTrueOffsetParent(element); // Find the nearest non-table offsetParent

    while (offsetParent && isTableElement(offsetParent)) {
      offsetParent = getTrueOffsetParent(offsetParent);
    }

    if (offsetParent && getNodeName(offsetParent) === 'body' && getComputedStyle(offsetParent).position === 'static') {
      return window;
    }

    return offsetParent || window;
  }

  var unwrapJqueryElement = (function (element) {
    return (// $FlowFixMe: need to get type refinement work
      element.hasOwnProperty('jquery') ? element[0] : element
    );
  });

  var order = function order(modifiers) {
    var map = new Map();
    var visited = new Set();
    var result = [];
    modifiers.forEach(function (modifier) {
      map.set(modifier.name, modifier);
    }); // On visiting object, check for its dependencies and visit them recursively

    function sort(modifier) {
      visited.add(modifier.name);
      var requires = [].concat(modifier.requires || [], modifier.optionallyRequires || []);
      requires.forEach(function (dep) {
        if (!visited.has(dep)) {
          var depModifier = map.get(dep);

          if (depModifier) {
            sort(depModifier);
          }
        }
      });
      result.push(modifier);
    }

    modifiers.forEach(function (modifier) {
      if (!visited.has(modifier.name)) {
        // check for visited object
        sort(modifier);
      }
    });
    return result;
  };

  var orderModifiers = (function (modifiers) {
    // order based on dependencies
    var orderedModifiers = order(modifiers); // order based on phase

    return modifierPhases.reduce(function (acc, phase) {
      return acc.concat(orderedModifiers.filter(function (modifier) {
        return modifier.phase === phase;
      }));
    }, []);
  });

  function microtaskDebounce(fn) {
    var pending;
    return function () {
      if (!pending) {
        pending = new Promise(function (resolve) {
          Promise.resolve().then(function () {
            pending = undefined;
            resolve(fn());
          });
        });
      }

      return pending;
    };
  }

  var format = (function (str) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return [].concat(args).reduce(function (p, c) {
      return p.replace(/%s/, c);
    }, str);
  });

  var INVALID_MODIFIER_ERROR = 'Popper: modifier "%s" provided an invalid %s property, expected %s but got %s';
  var MISSING_DEPENDENCY_ERROR = 'Popper: modifier "%s" requires "%s", but "%s" modifier is not available';
  var VALID_PROPERTIES = ['name', 'enabled', 'phase', 'fn', 'onLoad', 'requires', 'options'];
  var validateModifiers = (function (modifiers) {
    modifiers.forEach(function (modifier) {
      Object.keys(modifier).forEach(function (key) {
        switch (key) {
          case 'name':
            if (typeof modifier.name !== 'string') {
              console.error(format(INVALID_MODIFIER_ERROR, String(modifier.name), '"name"', '"string"', "\"" + String(modifier.name) + "\""));
            }

            break;

          case 'enabled':
            if (typeof modifier.enabled !== 'boolean') {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"enabled"', '"boolean"', "\"" + String(modifier.enabled) + "\""));
            }

          case 'phase':
            if (!modifierPhases.includes(modifier.phase)) {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"phase"', "either " + modifierPhases.join(', '), "\"" + String(modifier.phase) + "\""));
            }

            break;

          case 'fn':
            if (typeof modifier.fn !== 'function') {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"fn"', '"function"', "\"" + String(modifier.fn) + "\""));
            }

            break;

          case 'onLoad':
            if (typeof modifier.onLoad !== 'function') {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"onLoad"', '"function"', "\"" + String(modifier.fn) + "\""));
            }

            break;

          case 'onDestroy':
            if (typeof modifier.onDestroy !== 'function') {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"onDestroy"', '"function"', "\"" + String(modifier.fn) + "\""));
            }

            break;

          case 'requires':
            if (!Array.isArray(modifier.requires)) {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"requires"', '"array"', "\"" + String(modifier.requires) + "\""));
            }

            break;

          case 'optionallyRequires':
            if (!Array.isArray(modifier.requires)) {
              console.error(format(INVALID_MODIFIER_ERROR, modifier.name, '"requires"', '"array"', "\"" + String(modifier.requires) + "\""));
            }

            break;

          case 'options':
          case 'data':
            break;

          default:
            console.error("PopperJS: an invalid property has been provided to the \"" + modifier.name + "\" modifier, valid properties are " + VALID_PROPERTIES.map(function (s) {
              return "\"" + s + "\"";
            }).join(', ') + "; but \"" + key + "\" was provided.");
        }

        modifier.requires && modifier.requires.forEach(function (requirement) {
          if (modifiers.find(function (mod) {
            return mod.name === requirement;
          }) == null) {
            console.error(format(MISSING_DEPENDENCY_ERROR, String(modifier.name), requirement, requirement));
          }
        });
      });
    });
  });

  var INVALID_ELEMENT_ERROR = 'Popper: Invalid reference or popper argument provided to Popper, they must be either a valid DOM element, virtual element, or a jQuery-wrapped DOM element.';
  var INFINITE_LOOP_ERROR = 'Popper: An infinite loop in the modifiers cycle has been detected! The cycle has been interrupted to prevent a browser crash.';

  var areValidElements = function areValidElements() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return !args.some(function (element) {
      return !(element && typeof element.getBoundingClientRect === 'function');
    });
  };

  var defaultOptionsValue = {
    placement: 'bottom',
    modifiers: [],
    strategy: 'absolute'
  };
  function popperGenerator(generatorOptions) {
    if (generatorOptions === void 0) {
      generatorOptions = {};
    }

    var _generatorOptions = generatorOptions,
        _generatorOptions$def = _generatorOptions.defaultModifiers,
        defaultModifiers = _generatorOptions$def === void 0 ? [] : _generatorOptions$def,
        _generatorOptions$def2 = _generatorOptions.defaultOptions,
        defaultOptions = _generatorOptions$def2 === void 0 ? defaultOptionsValue : _generatorOptions$def2;
    return function createPopper(reference, popper, options) {
      if (options === void 0) {
        options = defaultOptions;
      }

      // Unwrap `reference` and `popper` elements in case they are
      // wrapped by jQuery, otherwise consume them as is
      var referenceElement = unwrapJqueryElement(reference);
      var popperElement = unwrapJqueryElement(popper);
      var state = {
        placement: 'bottom',
        orderedModifiers: [],
        options: Object.assign({}, defaultOptionsValue, {}, defaultOptions),
        modifiersData: {},
        elements: {
          reference: referenceElement,
          popper: popperElement
        },
        attributes: {},
        styles: {}
      };
      var instance = {
        setOptions: function setOptions(options) {
          // Store options into state
          state.options = Object.assign({}, defaultOptions, {}, options);
          state.scrollParents = {
            reference: listScrollParents(referenceElement),
            popper: listScrollParents(popperElement)
          }; // Order `options.modifiers` so that the dependencies are fulfilled
          // once the modifiers are executed

          state.orderedModifiers = orderModifiers([].concat(state.options.modifiers.filter(function (modifier) {
            return !defaultModifiers.find(function (_ref) {
              var name = _ref.name;
              return name === modifier.name;
            });
          }), defaultModifiers.map(function (defaultModifier) {
            return Object.assign({}, defaultModifier, {}, state.options.modifiers.find(function (_ref2) {
              var name = _ref2.name;
              return name === defaultModifier.name;
            }));
          }))); // Validate the provided modifiers so that the consumer will get warned
          // if one of the custom modifiers is invalid for any reason

          {
            validateModifiers(state.orderedModifiers);
          }
        },
        // Syncronous and forcefully executed update
        // it will always be executed even if not necessary, usually NOT needed
        // use Popper#update instead
        forceUpdate: function forceUpdate() {
          var _state$elements = state.elements,
              referenceElement = _state$elements.reference,
              popperElement = _state$elements.popper; // Don't proceed if `reference` or `popper` are not valid elements anymore

          if (!areValidElements(referenceElement, popperElement)) {
            {
              console.error(INVALID_ELEMENT_ERROR);
            }

            return;
          }

          var isFixed = state.options.strategy === 'fixed'; // Get initial measurements
          // these are going to be used to compute the initial popper offsets
          // and as cache for any modifier that needs them later

          state.measures = {
            reference: getCompositeRect(referenceElement, getOffsetParent(popperElement), isFixed),
            popper: getLayoutRect(popperElement)
          }; // Modifiers have the ability to read the current Popper state, included
          // the popper offsets, and modify it to address specifc cases

          state.reset = false; // Cache the placement in cache to make it available to the modifiers
          // modifiers will modify this one (rather than the one in options)

          state.placement = state.options.placement;
          state.orderedModifiers.forEach(function (modifier) {
            return state.modifiersData[modifier.name] = Object.assign({}, modifier.data);
          });
          var __debug_loops__ = 0;

          for (var index = 0; index < state.orderedModifiers.length; index++) {
            {
              __debug_loops__ += 1;

              if (__debug_loops__ > 100) {
                console.error(INFINITE_LOOP_ERROR);
                break;
              }
            }

            if (state.reset === true) {
              state.reset = false;
              index = -1;
              continue;
            }

            var _state$orderedModifie = state.orderedModifiers[index],
                fn = _state$orderedModifie.fn,
                enabled = _state$orderedModifie.enabled,
                _state$orderedModifie2 = _state$orderedModifie.options,
                _options = _state$orderedModifie2 === void 0 ? {} : _state$orderedModifie2,
                name = _state$orderedModifie.name;

            if (enabled && typeof fn === 'function') {
              state = fn({
                state: state,
                options: _options,
                name: name,
                instance: instance
              });
            }
          }
        },
        // Async and optimistically optimized update
        // it will not be executed if not necessary
        // debounced, so that it only runs at most once-per-tick
        update: microtaskDebounce(function () {
          return (// prettier-ignore
            new Promise(function (resolve) {
              instance.forceUpdate();
              resolve(state);
            })
          );
        }),
        destroy: function destroy() {
          // Run `onDestroy` modifier methods
          state.orderedModifiers.forEach(function (_ref3) {
            var onDestroy = _ref3.onDestroy,
                enabled = _ref3.enabled,
                name = _ref3.name,
                _ref3$options = _ref3.options,
                options = _ref3$options === void 0 ? {} : _ref3$options;
            return enabled && onDestroy && onDestroy({
              state: state,
              name: name,
              instance: instance,
              options: options
            });
          });
        }
      }; // Don't proceed if `reference` or `popper` are invalid elements

      if (!areValidElements(referenceElement, popperElement)) {
        {
          console.error(INVALID_ELEMENT_ERROR);
        }

        return instance;
      }

      instance.setOptions(options); // Modifiers have the opportunity to execute some arbitrary code before
      // the first update cycle is ran, the order of execution will be the same
      // defined by the modifier dependencies directive.
      // The `onLoad` function may add or alter the options of themselves

      state.orderedModifiers.forEach(function (_ref4) {
        var onLoad = _ref4.onLoad,
            enabled = _ref4.enabled,
            name = _ref4.name,
            _ref4$options = _ref4.options,
            options = _ref4$options === void 0 ? {} : _ref4$options;
        return enabled && onLoad && (state = onLoad({
          state: state,
          name: name,
          instance: instance,
          options: options
        }) || state);
      });
      instance.update();
      return instance;
    };
  }
  var createPopper = popperGenerator();

  var passive = {
    passive: true
  };

  function toggleEventListeners(_ref) {
    var state = _ref.state,
        instance = _ref.instance,
        scroll = _ref.scroll,
        resize = _ref.resize;

    if (scroll != null) {
      var scrollParents = [].concat(state.scrollParents.reference, state.scrollParents.popper);
      scrollParents.forEach(function (scrollParent) {
        return scroll ? scrollParent.addEventListener('scroll', instance.update, passive) : scrollParent.removeEventListener('scroll', instance.update);
      });
    }

    if (resize != null) {
      var window = getWindow(state.elements.popper);
      resize ? window.addEventListener('resize', instance.update, passive) : window.removeEventListener('resize', instance.update);
    }
  }

  function onLoad(_ref2) {
    var state = _ref2.state,
        instance = _ref2.instance,
        name = _ref2.name,
        options = _ref2.options;
    var _options$scroll = options.scroll,
        scroll = _options$scroll === void 0 ? true : _options$scroll,
        _options$resize = options.resize,
        resize = _options$resize === void 0 ? true : _options$resize; // cache initial options so we can compare them later

    state.modifiersData[name + "#persistent"] = {
      scroll: scroll,
      resize: resize
    };
    toggleEventListeners({
      state: state,
      instance: instance,
      scroll: scroll,
      resize: resize
    });
    return state;
  }

  function onDestroy(_ref3) {
    var state = _ref3.state,
        instance = _ref3.instance;
    toggleEventListeners({
      state: state,
      instance: instance,
      scroll: false,
      resize: false
    });
  }

  function update(_ref4) {
    var state = _ref4.state,
        options = _ref4.options,
        instance = _ref4.instance,
        name = _ref4.name;
    var data = state.modifiersData[name + "#persistent"];
    var _options$scroll2 = options.scroll,
        scroll = _options$scroll2 === void 0 ? true : _options$scroll2,
        _options$resize2 = options.resize,
        resize = _options$resize2 === void 0 ? true : _options$resize2; // set options to `null` if they didn't change, so we know not to run any logic

    if (data.scroll === scroll) {
      scroll = null;
    }

    if (data.resize === resize) {
      resize = null;
    } // Update cache


    state.modifiersData[name + "#persistent"] = {
      scroll: scroll,
      resize: resize
    };
    toggleEventListeners({
      state: state,
      instance: instance,
      scroll: scroll,
      resize: resize
    });
    return state;
  }

  var eventListeners = {
    name: 'eventListeners',
    enabled: true,
    phase: 'write',
    fn: update,
    onLoad: onLoad,
    onDestroy: onDestroy,
    data: {}
  };

  var getBasePlacement = (function (placement) {
    return placement.split('-')[0];
  });

  var getVariation = (function (placement) {
    return placement.split('-')[1];
  });

  var getMainAxisFromPlacement = (function (placement) {
    return ['top', 'bottom'].includes(placement) ? 'x' : 'y';
  });

  var computeOffsets = (function (_ref) {
    var reference = _ref.reference,
        element = _ref.element,
        placement = _ref.placement;
    var basePlacement = placement ? getBasePlacement(placement) : null;
    var variation = placement ? getVariation(placement) : null;
    var offsets;

    switch (basePlacement) {
      case top:
        offsets = {
          x: reference.x + reference.width / 2 - element.width / 2,
          y: reference.y - element.height
        };
        break;

      case bottom:
        offsets = {
          x: reference.x + reference.width / 2 - element.width / 2,
          y: reference.y + reference.height
        };
        break;

      case right:
        offsets = {
          x: reference.x + reference.width,
          y: reference.y + reference.height / 2 - element.height / 2
        };
        break;

      case left:
        offsets = {
          x: reference.x - element.width,
          y: reference.y + reference.height / 2 - element.height / 2
        };
        break;

      default:
        offsets = {
          x: reference.x,
          y: reference.y
        };
    }

    var mainAxis = basePlacement ? getMainAxisFromPlacement(basePlacement) : null;

    if (mainAxis != null) {
      var len = mainAxis === 'y' ? 'height' : 'width';

      switch (variation) {
        case start:
          offsets[mainAxis] = Math.floor(offsets[mainAxis]) - Math.floor(reference[len] / 2 - element[len] / 2);
          break;

        case end:
          offsets[mainAxis] = Math.floor(offsets[mainAxis]) + Math.ceil(reference[len] / 2 - element[len] / 2);
          break;
      }
    }

    return offsets;
  });

  function popperOffsets(_ref) {
    var state = _ref.state,
        name = _ref.name;
    // Offsets are the actual position the popper needs to have to be
    // properly positioned near its reference element
    // This is the most basic placement, and will be adjusted by
    // the modifiers in the next step
    state.modifiersData[name] = computeOffsets({
      reference: state.measures.reference,
      element: state.measures.popper,
      strategy: 'absolute',
      placement: state.placement
    });
    return state;
  }

  var popperOffsets$1 = {
    name: 'popperOffsets',
    enabled: true,
    phase: 'read',
    fn: popperOffsets,
    data: {}
  };

  // This modifier takes the Popper state and prepares some StyleSheet properties
  // that can be applied to the popper element to make it render in the expected
  // position.
  // Round the offsets to the nearest suitable subpixel based on the DPR.
  // Zooming can change the DPR, but it seems to report a value that will
  // cleanly divide the values into the appropriate subpixels.
  var roundOffsets = function roundOffsets(_ref) {
    var x = _ref.x,
        y = _ref.y;
    var dpr = window.devicePixelRatio;
    return {
      x: Math.round(x * dpr) / dpr || 0,
      y: Math.round(y * dpr) / dpr || 0
    };
  };

  var mapToStyles = function mapToStyles(_ref2) {
    var offsets = _ref2.offsets,
        position = _ref2.position,
        gpuAcceleration = _ref2.gpuAcceleration;

    var _roundOffsets = roundOffsets(offsets),
        x = _roundOffsets.x,
        y = _roundOffsets.y;

    var hasX = offsets.hasOwnProperty('x');
    var hasY = offsets.hasOwnProperty('y');

    if (gpuAcceleration) {
      return {
        top: hasY ? '0' : '',
        left: hasX ? '0' : '',
        // Layer acceleration can disable subpixel rendering which causes slightly
        // blurry text on low PPI displays, so we want to use 2D transforms
        // instead
        transform: window.devicePixelRatio < 2 ? "translate(" + x + "px, " + y + "px)" : "translate3d(" + x + "px, " + y + "px, 0)",
        position: position
      };
    }

    return {
      top: hasY ? y + "px" : '',
      left: hasX ? x + "px" : '',
      transform: '',
      position: position
    };
  };

  function computeStyles(_ref3) {
    var state = _ref3.state,
        options = _ref3.options;
    var _options$gpuAccelerat = options.gpuAcceleration,
        gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat; // popper offsets are always available

    state.styles.popper = Object.assign({}, state.styles.popper, {}, mapToStyles({
      offsets: state.modifiersData.popperOffsets,
      position: state.options.strategy,
      gpuAcceleration: gpuAcceleration
    })); // arrow offsets may not be available

    if (state.modifiersData.arrow != null) {
      state.styles.arrow = Object.assign({}, state.styles.arrow, {}, mapToStyles({
        offsets: state.modifiersData.arrow,
        position: 'absolute',
        gpuAcceleration: gpuAcceleration
      }));
    }

    state.attributes.popper = Object.assign({}, state.attributes.popper, {
      'data-popper-placement': state.placement
    });
    return state;
  }

  var computeStyles$1 = {
    name: 'computeStyles',
    enabled: true,
    phase: 'beforeWrite',
    fn: computeStyles,
    data: {}
  };

  // and applies them to the HTMLElements such as popper and arrow

  function applyStyles(_ref) {
    var state = _ref.state;
    Object.keys(state.elements).forEach(function (name) {
      var style = state.styles[name] || {};
      var attributes = state.attributes[name] || {};
      var element = state.elements[name]; // arrow is optional + virtual elements

      if (!isHTMLElement(element) || !getNodeName(element)) {
        return;
      } // Flow doesn't support to extend this property, but it's the most
      // effective way to apply styles to an HTMLElement
      // $FlowFixMe


      Object.assign(element.style, style);
      Object.entries(attributes).forEach(function (_ref2) {
        var name = _ref2[0],
            value = _ref2[1];

        if (value === false) {
          element.removeAttribute(name);
        } else {
          element.setAttribute(name, value === true ? '' : value);
        }
      });
    });
    return state;
  }

  function onDestroy$1(_ref3) {
    var state = _ref3.state;
    Object.keys(state.elements).forEach(function (name) {
      var element = state.elements[name];
      var styleProperties = Object.keys(state.styles.hasOwnProperty(name) ? Object.assign({}, state.styles[name]) : {});
      var attributes = state.attributes[name] || {}; // Set all values to an empty string to unset them

      var style = styleProperties.reduce(function (style, property) {
        var _Object$assign;

        return Object.assign({}, style, (_Object$assign = {}, _Object$assign[String(property)] = '', _Object$assign));
      }, {}); // arrow is optional + virtual elements

      if (!isHTMLElement(element) || !getNodeName(element)) {
        return;
      } // Flow doesn't support to extend this property, but it's the most
      // effective way to apply styles to an HTMLElement
      // $FlowFixMe


      Object.assign(element.style, style);
      Object.keys(attributes).forEach(function (attribute) {
        return element.removeAttribute(attribute);
      });
    });
  }

  var applyStyles$1 = {
    name: 'applyStyles',
    enabled: true,
    phase: 'write',
    fn: applyStyles,
    onDestroy: onDestroy$1,
    requires: ['computeStyles']
  };

  function distanceAndSkiddingToXY(placement, measures, offsetValue) {
    var basePlacement = getBasePlacement(placement);
    var invertDistance = [left, top].includes(basePlacement) ? -1 : 1;

    var _ref = typeof offsetValue === 'function' ? offsetValue(Object.assign({}, measures, {
      placement: placement
    })) : offsetValue,
        distance = _ref[0],
        skidding = _ref[1];

    distance = (distance || 0) * invertDistance;
    skidding = skidding || 0;
    return [left, right].includes(basePlacement) ? {
      x: distance,
      y: skidding
    } : {
      x: skidding,
      y: distance
    };
  }

  function offset(_ref2) {
    var state = _ref2.state,
        options = _ref2.options,
        name = _ref2.name;
    var _options$offset = options.offset,
        offset = _options$offset === void 0 ? [0, 0] : _options$offset;
    var data = placements.reduce(function (acc, placement) {
      acc[placement] = distanceAndSkiddingToXY(placement, state.measures, offset);
      return acc;
    }, {});
    var _data$state$placement = data[state.placement],
        x = _data$state$placement.x,
        y = _data$state$placement.y;
    state.modifiersData.popperOffsets.x += x;
    state.modifiersData.popperOffsets.y += y;
    state.modifiersData[name] = data;
    return state;
  }

  var offset$1 = {
    name: 'offset',
    enabled: true,
    phase: 'main',
    requires: ['popperOffsets'],
    fn: offset
  };

  var hash = {
    left: 'right',
    right: 'left',
    bottom: 'top',
    top: 'bottom'
  };
  var getOppositePlacement = (function (placement) {
    return placement.replace(/left|right|bottom|top/g, function (matched) {
      return hash[matched];
    });
  });

  var hash$1 = {
    start: 'end',
    end: 'start'
  };
  var getOppositeVariationPlacement = (function (placement) {
    return placement.replace(/start|end/g, function (matched) {
      return hash$1[matched];
    });
  });

  var getFreshSideObject = (function () {
    return {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    };
  });

  var mergePaddingObject = (function (paddingObject) {
    return Object.assign({}, getFreshSideObject(), {}, paddingObject);
  });

  function expandToHashMap(value, keys) {
    return keys.reduce(function (hashMap, key) {
      hashMap[key] = value;
      return hashMap;
    }, {});
  }

  var getViewportRect = (function (element) {
    var win = getWindow(element);
    return {
      width: win.innerWidth,
      height: win.innerHeight,
      x: 0,
      y: 0
    };
  });

  var getDocumentElement = (function (element) {
    return element.ownerDocument.documentElement;
  });

  var getDocumentRect = (function (element) {
    var win = getWindow(element);
    var winScroll = getWindowScroll(element);
    var documentRect = getCompositeRect(getDocumentElement(element), win);
    documentRect.height = Math.max(documentRect.height, win.innerHeight);
    documentRect.width = Math.max(documentRect.width, win.innerWidth);
    documentRect.x = -winScroll.scrollLeft;
    documentRect.y = -winScroll.scrollTop;
    return documentRect;
  });

  var rectToClientRect = (function (rect) {
    return Object.assign({}, rect, {
      left: rect.x,
      top: rect.y,
      right: rect.x + rect.width,
      bottom: rect.y + rect.height
    });
  });

  function getClientRectFromMixedType(element, clippingParent) {
    return clippingParent === viewport ? rectToClientRect(getViewportRect(element)) : isHTMLElement(clippingParent) ? getBoundingClientRect(clippingParent) : rectToClientRect(getDocumentRect(getDocumentElement(element)));
  } // A "clipping parent" is a scrolling container with the characteristic of
  // clipping (or hiding) overflowing elements with a position different from
  // `initial`


  function getClippingParents(element) {
    var scrollParents = listScrollParents(element);
    var canEscapeClipping = ['absolute', 'fixed'].includes(getComputedStyle(element).position);
    var clipperElement = canEscapeClipping && isHTMLElement(element) ? getOffsetParent(element) : element;

    if (!isElement(clipperElement)) {
      return [];
    }

    return scrollParents.filter(function (scrollParent) {
      return isElement(scrollParent) && scrollParent.contains(clipperElement);
    });
  } // Gets the maximum area that the element is visible in due to any number of
  // clipping parents


  function getClippingRect(element, boundary, rootBoundary) {
    var mainClippingParents = boundary === 'clippingParents' ? getClippingParents(element) : [].concat(boundary);
    var clippingParents = [].concat(mainClippingParents, [rootBoundary]);
    var firstClippingParent = clippingParents[0];
    var clippingRect = clippingParents.reduce(function (accRect, clippingParent) {
      var rect = getClientRectFromMixedType(element, clippingParent);
      var borders = isHTMLElement(clippingParent) ? getBorders(clippingParent) : getFreshSideObject();
      accRect.top = Math.max(rect.top + borders.top, accRect.top);
      accRect.right = Math.min(rect.right - borders.right, accRect.right);
      accRect.bottom = Math.min(rect.bottom - borders.bottom, accRect.bottom);
      accRect.left = Math.max(rect.left + borders.left, accRect.left);
      return accRect;
    }, getClientRectFromMixedType(element, firstClippingParent));
    clippingRect.width = clippingRect.right - clippingRect.left;
    clippingRect.height = clippingRect.bottom - clippingRect.top;
    clippingRect.x = clippingRect.left;
    clippingRect.y = clippingRect.top;
    return clippingRect;
  }

  var unwrapVirtualElement = (function (element) {
    return isElement(element) ? element : element.contextElement;
  });

  function detectOverflow(state, options) {
    if (options === void 0) {
      options = {};
    }

    var _options = options,
        _options$placement = _options.placement,
        placement = _options$placement === void 0 ? state.placement : _options$placement,
        _options$boundary = _options.boundary,
        boundary = _options$boundary === void 0 ? clippingParents : _options$boundary,
        _options$rootBoundary = _options.rootBoundary,
        rootBoundary = _options$rootBoundary === void 0 ? 'document' : _options$rootBoundary,
        _options$elementConte = _options.elementContext,
        elementContext = _options$elementConte === void 0 ? popper : _options$elementConte,
        _options$altBoundary = _options.altBoundary,
        altBoundary = _options$altBoundary === void 0 ? false : _options$altBoundary;
    var altContext = elementContext === popper ? reference : popper;
    var referenceElement = state.elements.reference;
    var popperRect = state.measures.popper;
    var element = state.elements[altBoundary ? altContext : elementContext];
    var clippingClientRect = getClippingRect(unwrapVirtualElement(element), boundary, rootBoundary);
    var referenceClientRect = getBoundingClientRect(referenceElement);
    var popperOffsets = computeOffsets({
      reference: referenceClientRect,
      element: popperRect,
      strategy: 'absolute',
      placement: placement
    });
    var popperClientRect = rectToClientRect(Object.assign({}, popperRect, {}, popperOffsets));
    var elementClientRect = elementContext === popper ? popperClientRect : referenceClientRect; // positive = overflowing the clipping rect
    // 0 or negative = within the clipping rect

    var overflowOffsets = {
      top: clippingClientRect.top - elementClientRect.top,
      bottom: elementClientRect.bottom - clippingClientRect.bottom,
      left: clippingClientRect.left - elementClientRect.left,
      right: elementClientRect.right - clippingClientRect.right
    };
    var offsetData = state.modifiersData.offset; // Offsets can be applied only to the popper element

    if (elementContext === popper && offsetData) {
      var offset = offsetData[placement];
      Object.keys(overflowOffsets).forEach(function (key) {
        var multiply = [right, bottom].includes(key) ? 1 : -1;
        var axis = [top, bottom].includes(key) ? 'y' : 'x';
        overflowOffsets[key] += offset[axis] * multiply;
      });
    }

    return overflowOffsets;
  }

  var getExpandedFallbackPlacements = function getExpandedFallbackPlacements(placement) {
    var oppositePlacement = getOppositePlacement(placement);
    return [getOppositeVariationPlacement(placement), oppositePlacement, getOppositeVariationPlacement(oppositePlacement)];
  };

  function flip(_ref) {
    var state = _ref.state,
        options = _ref.options,
        name = _ref.name;
    var specifiedFallbackPlacements = options.fallbackPlacements,
        _options$padding = options.padding,
        padding = _options$padding === void 0 ? 0 : _options$padding,
        _options$boundary = options.boundary,
        boundary = _options$boundary === void 0 ? clippingParents : _options$boundary,
        _options$rootBoundary = options.rootBoundary,
        rootBoundary = _options$rootBoundary === void 0 ? viewport : _options$rootBoundary,
        _options$flipVariatio = options.flipVariations,
        flipVariations = _options$flipVariatio === void 0 ? true : _options$flipVariatio;
    var preferredPlacement = state.options.placement;
    var isBasePlacement = getBasePlacement(preferredPlacement) === preferredPlacement;
    var fallbackPlacements = specifiedFallbackPlacements || (isBasePlacement ? [getOppositePlacement(preferredPlacement)] : getExpandedFallbackPlacements(preferredPlacement));
    var flipIndex = state.modifiersData[name].index;
    var paddingObject = mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements));
    var placementOrder = [preferredPlacement].concat(fallbackPlacements);
    var flippedPlacement = placementOrder[flipIndex];
    var referenceRect = state.measures.reference;
    var popperRect = state.measures.popper;
    var overflow = detectOverflow(state, {
      placement: flippedPlacement,
      boundary: boundary,
      rootBoundary: rootBoundary
    });

    if (!flippedPlacement) {
      // If none of the placements fit, we still want to use the better-fitting
      // flipped variation if possible
      var fallbackVariation = state.modifiersData[name].fallbackVariation;

      if (fallbackVariation && state.placement !== fallbackVariation) {
        state.placement = fallbackVariation;
        state.reset = true;
      }

      return state;
    }

    var basePlacement = getBasePlacement(flippedPlacement);
    var variation = getVariation(flippedPlacement);
    var isVertical = [top, bottom].includes(basePlacement);
    var isStartVariation = variation === start;
    var len = isVertical ? 'width' : 'height';
    var edge = referenceRect[len] < popperRect[len] ? isVertical ? isStartVariation ? right : left : isStartVariation ? bottom : top : isVertical ? isStartVariation ? left : right : isStartVariation ? top : bottom;
    var fits = overflow[basePlacement] + paddingObject[basePlacement] <= 0;

    if (flipVariations && !isBasePlacement) {
      var fitsEdge = overflow[edge] + paddingObject[edge] <= 0;
      fits = fits && fitsEdge;

      if (fitsEdge && !state.modifiersData[name].fallbackVariation) {
        state.modifiersData[name].fallbackVariation = flippedPlacement;
      }
    }

    if (!fits) {
      state.modifiersData[name].index += 1;
      state.reset = true;
      return state;
    } else if (fits && state.placement !== flippedPlacement) {
      state.placement = flippedPlacement;
      state.reset = true;
      return state;
    }

    return state;
  }

  var flip$1 = {
    name: 'flip',
    enabled: true,
    phase: 'main',
    fn: flip,
    requires: [],
    optionallyRequires: ['offset'],
    data: {
      index: 0
    }
  };

  var getAltAxis = (function (axis) {
    return axis === 'x' ? 'y' : 'x';
  });

  var within = (function (min, value, max) {
    return Math.max(min, Math.min(value, max));
  });

  function preventOverflow(_ref) {
    var state = _ref.state,
        options = _ref.options,
        name = _ref.name;
    var _options$mainAxis = options.mainAxis,
        checkMainAxis = _options$mainAxis === void 0 ? true : _options$mainAxis,
        _options$altAxis = options.altAxis,
        checkAltAxis = _options$altAxis === void 0 ? false : _options$altAxis,
        _options$boundary = options.boundary,
        boundary = _options$boundary === void 0 ? clippingParents : _options$boundary,
        _options$rootBoundary = options.rootBoundary,
        rootBoundary = _options$rootBoundary === void 0 ? 'document' : _options$rootBoundary,
        _options$tether = options.tether,
        tether = _options$tether === void 0 ? true : _options$tether,
        _options$tetherOffset = options.tetherOffset,
        tetherOffset = _options$tetherOffset === void 0 ? 0 : _options$tetherOffset,
        _options$padding = options.padding,
        padding = _options$padding === void 0 ? 0 : _options$padding;
    var overflow = detectOverflow(state, {
      boundary: boundary,
      rootBoundary: rootBoundary
    });
    var basePlacement = getBasePlacement(state.placement);
    var variation = getVariation(state.placement);
    var isBasePlacement = !variation;
    var mainAxis = getMainAxisFromPlacement(basePlacement);
    var altAxis = getAltAxis(mainAxis);
    var popperOffsets = state.modifiersData.popperOffsets;
    var referenceRect = state.measures.reference;
    var popperRect = state.measures.popper;
    var paddingObject = mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements));
    var tetherOffsetValue = typeof tetherOffset === 'function' ? tetherOffset(Object.assign({}, state.measures, {
      placement: state.placement
    })) : tetherOffset;
    var data = {
      x: 0,
      y: 0
    };

    if (checkMainAxis) {
      var mainSide = mainAxis === 'y' ? top : left;
      var altSide = mainAxis === 'y' ? bottom : right;
      var len = mainAxis === 'y' ? 'height' : 'width';
      var offset = popperOffsets[mainAxis];
      var min = popperOffsets[mainAxis] + overflow[mainSide] + paddingObject[mainSide];
      var max = popperOffsets[mainAxis] - overflow[altSide] - paddingObject[altSide];
      var additive = tether ? -popperRect[len] / 2 : 0;
      var minLen = variation === start ? referenceRect[len] : popperRect[len];
      var maxLen = variation === start ? -popperRect[len] : -referenceRect[len]; // We need to include the arrow in the calculation so the arrow doesn't go
      // outside the reference bounds

      var arrowElement = state.elements.arrow;
      var arrowRect = tether && arrowElement ? getLayoutRect(arrowElement) : {
        width: 0,
        height: 0
      };
      var arrowPaddingObject = state.modifiersData['arrow#persistent'] ? state.modifiersData['arrow#persistent'].padding : getFreshSideObject();
      var arrowPaddingMin = arrowPaddingObject ? arrowPaddingObject[mainSide] : 0;
      var arrowPaddingMax = arrowPaddingObject ? arrowPaddingObject[altSide] : 0; // If the reference length is smaller than the arrow length, we don't want
      // to include its full size in the calculation. If the reference is small
      // and near the edge of a boundary, the popper can overflow even if the
      // reference is not overflowing as well (e.g. virtual elements with no
      // width or height)

      var arrowLen = within(0, Math.abs(referenceRect[len] - arrowRect[len]), arrowRect[len]);
      var minOffset = isBasePlacement ? referenceRect[len] / 2 - additive - arrowLen - arrowPaddingMin - tetherOffsetValue : minLen - arrowLen - arrowPaddingMin - tetherOffsetValue;
      var maxOffset = isBasePlacement ? -referenceRect[len] / 2 + additive + arrowLen + arrowPaddingMax + tetherOffsetValue : maxLen + arrowLen + arrowPaddingMax + tetherOffsetValue;
      var tetherMin = state.modifiersData.popperOffsets[mainAxis] + minOffset;
      var tetherMax = state.modifiersData.popperOffsets[mainAxis] + maxOffset;
      var preventedOffset = within(tether ? Math.min(min, tetherMin) : min, offset, tether ? Math.max(max, tetherMax) : max);
      state.modifiersData.popperOffsets[mainAxis] = preventedOffset;
      data[mainAxis] = preventedOffset - offset;
    }

    if (checkAltAxis) {
      var _mainSide = mainAxis === 'x' ? top : left;

      var _altSide = mainAxis === 'x' ? bottom : right;

      var _offset = popperOffsets[altAxis];

      var _min = _offset + overflow[_mainSide] + paddingObject[_mainSide];

      var _max = _offset - overflow[_altSide] - paddingObject[_altSide];

      var _preventedOffset = within(_min, _offset, _max);

      state.modifiersData.popperOffsets[altAxis] = _preventedOffset;
      data[altAxis] = _preventedOffset - _offset;
    }

    state.modifiersData[name] = data;
    return state;
  }

  var preventOverflow$1 = {
    name: 'preventOverflow',
    enabled: true,
    phase: 'main',
    fn: preventOverflow,
    requires: [],
    optionallyRequires: ['offset']
  };

  function arrow(_ref) {
    var _state$modifiersData$;

    var state = _ref.state,
        name = _ref.name;
    var arrowElement = state.elements.arrow;
    var popperOffsets = state.modifiersData.popperOffsets;
    var basePlacement = getBasePlacement(state.placement);
    var axis = getMainAxisFromPlacement(basePlacement);
    var isVertical = [left, right].includes(basePlacement);
    var len = isVertical ? 'height' : 'width';

    if (!arrowElement) {
      return state;
    }

    var paddingObject = state.modifiersData[name + "#persistent"].padding;
    var arrowRect = getLayoutRect(arrowElement);
    var endDiff = state.measures.reference[len] + state.measures.reference[axis] - popperOffsets[axis] - state.measures.popper[len];
    var startDiff = popperOffsets[axis] - state.measures.reference[axis];
    var centerToReference = endDiff / 2 - startDiff / 2;
    var center = state.measures.popper[len] / 2 - arrowRect[len] / 2 + centerToReference;
    var minProp = axis === 'y' ? top : left;
    var maxProp = axis === 'y' ? bottom : right; // Make sure the arrow doesn't overflow the popper if the center point is
    // outside of the popper bounds

    center = within(paddingObject[minProp], center, state.measures.popper[len] - arrowRect[len] - paddingObject[maxProp]); // Prevents breaking syntax highlighting...

    var axisProp = axis;
    state.modifiersData[name] = (_state$modifiersData$ = {}, _state$modifiersData$[axisProp] = center, _state$modifiersData$);
    return state;
  }

  function onLoad$1(_ref2) {
    var state = _ref2.state,
        options = _ref2.options,
        name = _ref2.name;
    var _options$element = options.element,
        arrowElement = _options$element === void 0 ? '[data-popper-arrow]' : _options$element,
        _options$padding = options.padding,
        padding = _options$padding === void 0 ? 0 : _options$padding; // CSS selector

    if (typeof arrowElement === 'string') {
      arrowElement = state.elements.popper.querySelector(arrowElement);

      if (!arrowElement) {
        return state;
      }
    }

    if (!state.elements.popper.contains(arrowElement)) {
      {
        console.error(['Popper: "arrow" modifier\'s `element` must be a child of the popper', 'element.'].join(' '));
      }

      return state;
    }

    state.elements.arrow = arrowElement;
    state.modifiersData[name + "#persistent"] = {
      padding: mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements))
    };
    return state;
  }

  var arrow$1 = {
    name: 'arrow',
    enabled: true,
    phase: 'main',
    fn: arrow,
    onLoad: onLoad$1,
    requires: ['popperOffsets'],
    optionallyRequires: ['preventOverflow']
  };

  var getOffsets = function getOffsets(overflow, rect, preventedOffsets) {
    if (preventedOffsets === void 0) {
      preventedOffsets = {
        x: 0,
        y: 0
      };
    }

    return {
      top: overflow.top - rect.height - preventedOffsets.y,
      right: overflow.right - rect.width + preventedOffsets.x,
      bottom: overflow.bottom - rect.height + preventedOffsets.y,
      left: overflow.left - rect.width - preventedOffsets.x
    };
  };

  var isAnySideFullyClipped = function isAnySideFullyClipped(overflow) {
    return [top, right, bottom, left].some(function (side) {
      return overflow[side] >= 0;
    });
  };

  function hide(_ref) {
    var state = _ref.state,
        name = _ref.name;
    var referenceRect = state.measures.reference;
    var popperRect = state.measures.popper;
    var preventedOffsets = state.modifiersData.preventOverflow;
    var referenceOverflow = detectOverflow(state, {
      elementContext: 'reference'
    });
    var popperAltOverflow = detectOverflow(state, {
      altBoundary: true
    });
    var referenceClippingOffsets = getOffsets(referenceOverflow, referenceRect);
    var popperEscapeOffsets = getOffsets(popperAltOverflow, popperRect, preventedOffsets);
    var isReferenceHidden = isAnySideFullyClipped(referenceClippingOffsets);
    var hasPopperEscaped = isAnySideFullyClipped(popperEscapeOffsets);
    state.modifiersData[name] = {
      referenceClippingOffsets: referenceClippingOffsets,
      popperEscapeOffsets: popperEscapeOffsets,
      isReferenceHidden: isReferenceHidden,
      hasPopperEscaped: hasPopperEscaped
    };
    state.attributes.popper = Object.assign({}, state.attributes.popper, {
      'data-popper-reference-hidden': isReferenceHidden,
      'data-popper-escaped': hasPopperEscaped
    });
    return state;
  }

  var hide$1 = {
    name: 'hide',
    enabled: true,
    phase: 'main',
    requires: [],
    optionallyRequires: ['preventOverflow'],
    fn: hide
  };

  var defaultModifiers = [eventListeners, popperOffsets$1, computeStyles$1, applyStyles$1, offset$1, flip$1, preventOverflow$1, arrow$1, hide$1];
  var createPopper$1 = popperGenerator({
    defaultModifiers: defaultModifiers
  }); // eslint-disable-next-line import/no-unused-modules

  exports.createPopper = createPopper$1;
  exports.popperGenerator = popperGenerator;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=popper.js.map
