import getOppositePlacement from "../utils/getOppositePlacement.js";
import getBasePlacement from "../utils/getBasePlacement.js";
import getVariation from "../utils/getVariation.js";
import getOppositeVariationPlacement from "../utils/getOppositeVariationPlacement.js";
import mergePaddingObject from "../utils/mergePaddingObject.js";
import expandToHashMap from "../utils/expandToHashMap.js";
import { basePlacements, start, top, bottom, right, left, viewport, clippingParents } from "../enums.js";
import detectOverflow from "../utils/detectOverflow.js";

var getExpandedFallbackPlacements = function getExpandedFallbackPlacements(placement) {
  var oppositePlacement = getOppositePlacement(placement);
  return [getOppositeVariationPlacement(placement), oppositePlacement, getOppositeVariationPlacement(oppositePlacement)];
};

function flip(_ref) {
  var state = _ref.state,
      options = _ref.options,
      name = _ref.name;
  var specifiedFallbackPlacements = options.fallbackPlacements,
      _options$padding = options.padding,
      padding = _options$padding === void 0 ? 0 : _options$padding,
      _options$boundary = options.boundary,
      boundary = _options$boundary === void 0 ? clippingParents : _options$boundary,
      _options$rootBoundary = options.rootBoundary,
      rootBoundary = _options$rootBoundary === void 0 ? viewport : _options$rootBoundary,
      _options$flipVariatio = options.flipVariations,
      flipVariations = _options$flipVariatio === void 0 ? true : _options$flipVariatio;
  var preferredPlacement = state.options.placement;
  var isBasePlacement = getBasePlacement(preferredPlacement) === preferredPlacement;
  var fallbackPlacements = specifiedFallbackPlacements || (isBasePlacement ? [getOppositePlacement(preferredPlacement)] : getExpandedFallbackPlacements(preferredPlacement));
  var flipIndex = state.modifiersData[name].index;
  var paddingObject = mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements));
  var placementOrder = [preferredPlacement].concat(fallbackPlacements);
  var flippedPlacement = placementOrder[flipIndex];
  var referenceRect = state.measures.reference;
  var popperRect = state.measures.popper;
  var overflow = detectOverflow(state, {
    placement: flippedPlacement,
    boundary: boundary,
    rootBoundary: rootBoundary
  });

  if (!flippedPlacement) {
    // If none of the placements fit, we still want to use the better-fitting
    // flipped variation if possible
    var fallbackVariation = state.modifiersData[name].fallbackVariation;

    if (fallbackVariation && state.placement !== fallbackVariation) {
      state.placement = fallbackVariation;
      state.reset = true;
    }

    return state;
  }

  var basePlacement = getBasePlacement(flippedPlacement);
  var variation = getVariation(flippedPlacement);
  var isVertical = [top, bottom].includes(basePlacement);
  var isStartVariation = variation === start;
  var len = isVertical ? 'width' : 'height';
  var edge = referenceRect[len] < popperRect[len] ? isVertical ? isStartVariation ? right : left : isStartVariation ? bottom : top : isVertical ? isStartVariation ? left : right : isStartVariation ? top : bottom;
  var fits = overflow[basePlacement] + paddingObject[basePlacement] <= 0;

  if (flipVariations && !isBasePlacement) {
    var fitsEdge = overflow[edge] + paddingObject[edge] <= 0;
    fits = fits && fitsEdge;

    if (fitsEdge && !state.modifiersData[name].fallbackVariation) {
      state.modifiersData[name].fallbackVariation = flippedPlacement;
    }
  }

  if (!fits) {
    state.modifiersData[name].index += 1;
    state.reset = true;
    return state;
  } else if (fits && state.placement !== flippedPlacement) {
    state.placement = flippedPlacement;
    state.reset = true;
    return state;
  }

  return state;
}

export default {
  name: 'flip',
  enabled: true,
  phase: 'main',
  fn: flip,
  requires: [],
  optionallyRequires: ['offset'],
  data: {
    index: 0
  }
};